@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,8 +22,7 @@ Constraint::Constraint()
 {
 	ref_table=nullptr;
 	obj_type=ObjectType::Constraint;
-	deferrable=false;
-	no_inherit=false;
+	deferrable=no_inherit=nulls_not_distinct = false;
 	fill_factor=0;
 	index_type=IndexingType::Null;
 
@@ -48,6 +47,7 @@ Constraint::Constraint()
 	attributes[Attributes::Factor]="";
 	attributes[Attributes::NoInherit]="";
 	attributes[Attributes::Elements]="";
+	attributes[Attributes::NullsNotDistinct]="";
 }
 
 Constraint::~Constraint()
@@ -121,11 +121,11 @@ bool Constraint::isColumnsExist(std::vector<Column *> columns, ColumnsId cols_id
 	{
 		is_ref = isColumnExists(col, cols_id);
 		if(!strict_check && !is_ref) break;
-		if(strict_check) found_cols++;
+		if(strict_check && is_ref) found_cols++;
 	}
 
 	return (!strict_check && is_ref) ||
-					(strict_check && found_cols == getColumnCount(cols_id));
+				 (strict_check && found_cols == getColumnCount(cols_id));
 }
 
 bool Constraint::isColumnReferenced(Column *column, bool search_only_ref_cols)
@@ -190,10 +190,10 @@ void Constraint::addColumns(const std::vector<Column *> &cols, ColumnsId cols_id
 {
 	try
 	{
-		if(cols_id == ReferencedCols)
-			ref_columns.clear();
-		else
+		if(cols_id == SourceCols)
 			columns.clear();
+		else
+			ref_columns.clear();
 
 		for(auto &col : cols)
 			addColumn(col, cols_id);
@@ -264,9 +264,9 @@ void Constraint::setColumnsAttribute(ColumnsId cols_id, unsigned def_type, bool
 	attributes[attrib]=str_cols;
 }
 
-void Constraint::setReferencedTable(BaseTable *tab_ref)
+void Constraint::setReferencedTable(BaseTable *ref_tab)
 {
-	this->ref_table=tab_ref;
+	this->ref_table = ref_tab;
 }
 
 void Constraint::setDeferralType(DeferralType deferral_type)
@@ -522,18 +522,22 @@ std::vector<ExcludeElement> Constraint::getExcludeElements()
 
 void Constraint::addExcludeElements(std::vector<ExcludeElement> &elems)
 {
-	std::vector<ExcludeElement> elems_bkp=excl_elements;
+	std::vector<ExcludeElement> elems_bkp = excl_elements;
 
 	try
 	{
-		excl_elements.clear();
+		removeExcludeElements();
 
-		for(unsigned i=0; i < elems.size(); i++)
-			addExcludeElement(elems[i]);
+		for(auto &elem : elems)
+			addExcludeElement(elem);
 	}
 	catch(Exception &e)
 	{
-		excl_elements = elems_bkp;
+		removeExcludeElements();
+
+		for(auto &elem : elems_bkp)
+			addExcludeElement(elem);
+
 		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 	}
 }
@@ -550,78 +554,6 @@ void Constraint::addExcludeElement(ExcludeElement elem)
 	setCodeInvalidated(true);
 }
 
-void Constraint::addExcludeElement(const QString &expr, Operator *oper, OperatorClass *op_class, bool use_sorting, bool asc_order, bool nulls_first)
-{
-	try
-	{
-		ExcludeElement elem;
-
-		//Raises an error if the expression is empty
-		if(expr.isEmpty())
-			throw Exception(ErrorCode::AsgInvalidExpressionObject,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
-		//Configures the element
-		elem.setExpression(expr);
-		elem.setOperatorClass(op_class);
-		elem.setOperator(oper);
-		elem.setSortingEnabled(use_sorting);
-		elem.setSortingAttribute(ExcludeElement::NullsFirst, nulls_first);
-		elem.setSortingAttribute(ExcludeElement::AscOrder, asc_order);
-
-		if(getExcludeElementIndex(elem) >= 0)
-			throw Exception(ErrorCode::InsDuplicatedElement,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
-		excl_elements.push_back(elem);
-		setCodeInvalidated(true);
-	}
-	catch(Exception &e)
-	{
-		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
-	}
-}
-
-void Constraint::addExcludeElement(Column *column, Operator *oper, OperatorClass *op_class, bool use_sorting, bool asc_order, bool nulls_first)
-{
-	try
-	{
-		ExcludeElement elem;
-
-		//Case the column is not allocated raises an error
-		if(!column)
-			throw Exception(Exception::getErrorMessage(ErrorCode::AsgNotAllocatedColumn)
-							.arg(this->getName())
-							.arg(this->getTypeName()),
-							ErrorCode::AsgNotAllocatedColumn,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
-		//Configures the element
-		elem.setColumn(column);
-		elem.setOperatorClass(op_class);
-		elem.setOperator(oper);
-		elem.setSortingEnabled(use_sorting);
-		elem.setSortingAttribute(ExcludeElement::NullsFirst, nulls_first);
-		elem.setSortingAttribute(ExcludeElement::AscOrder, asc_order);
-
-		if(getExcludeElementIndex(elem) >= 0)
-			throw Exception(ErrorCode::InsDuplicatedElement,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
-		excl_elements.push_back(elem);
-		setCodeInvalidated(true);
-	}
-	catch(Exception &e)
-	{
-		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
-	}
-}
-
-void Constraint::removeExcludeElement(unsigned elem_idx)
-{
-	if(elem_idx >= excl_elements.size())
-		throw Exception(ErrorCode::RefElementInvalidIndex,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
-	excl_elements.erase(excl_elements.begin() + elem_idx);
-	setCodeInvalidated(true);
-}
-
 void Constraint::removeExcludeElements()
 {
 	excl_elements.clear();
@@ -633,13 +565,21 @@ void Constraint::setColumnsNotNull(bool value)
 	if(constr_type==ConstraintType::PrimaryKey)
 	{
 		for(auto &col : columns)
-		{
-			//if(!col->isAddedByRelationship())
 			col->setNotNull(value);
-		}
 	}
 }
 
+void Constraint::setNullsNotDistinct(bool value)
+{
+	setCodeInvalidated(nulls_not_distinct != value);
+	nulls_not_distinct = value;
+}
+
+bool Constraint::isNullsNotDistinct()
+{
+	return nulls_not_distinct;
+}
+
 ExcludeElement Constraint::getExcludeElement(unsigned elem_idx)
 {
 	if(elem_idx >= excl_elements.size())
@@ -764,6 +704,7 @@ QString Constraint::getSourceCode(SchemaParser::CodeType def_type, bool inc_adde
 	attributes[Attributes::RefTable]=(ref_table ? ref_table->getName(true) : "");
 	attributes[Attributes::Deferrable]=(deferrable ? Attributes::True : "");
 	attributes[Attributes::NoInherit]=(no_inherit ? Attributes::True : "");
+	attributes[Attributes::NullsNotDistinct]=(nulls_not_distinct ? Attributes::True : "");
 	attributes[Attributes::ComparisonType]=(~match_type);
 	attributes[Attributes::DeferType]=(~deferral_type);
 	attributes[Attributes::IndexType]=(~ index_type);
@@ -835,3 +776,20 @@ bool Constraint::isCodeDiffersFrom(BaseObject *object, const QStringList &ignore
 		throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 	}
 }
+
+void Constraint::updateDependencies()
+{
+	std::vector<BaseObject *> deps, aux_deps;
+
+	deps.insert(deps.end(), columns.begin(), columns.end());
+	deps.insert(deps.end(), ref_columns.begin(), ref_columns.end());
+	deps.push_back(ref_table);
+
+	for(auto &elem : excl_elements)
+	{
+		aux_deps = elem.getDependencies();
+		deps.insert(deps.end(), aux_deps.begin(), aux_deps.end());
+	}
+
+	TableObject::updateDependencies(deps);
+}
