@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -500,31 +500,12 @@ int OperationList::registerObject(BaseObject *object, Operation::OperType op_typ
 	}
 }
 
-void OperationList::getOperationData(unsigned oper_idx, unsigned &oper_type, QString &obj_name, ObjectType &obj_type)
+const Operation *OperationList::getOperation(unsigned oper_idx)
 {
-	Operation *operation=nullptr;
-	BaseObject *pool_obj=nullptr;
-
 	if(oper_idx >= operations.size())
 		throw Exception(ErrorCode::RefObjectInvalidIndex,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
-	operation=operations[oper_idx];
-	oper_type=operation->getOperationType();
-
-	if(operation->isOperationValid())
-	{
-		pool_obj=operation->getPoolObject();
-		obj_type=pool_obj->getObjectType();
-		obj_name=pool_obj->getName(true);
-
-		if(TableObject::isTableObject(obj_type))
-			obj_name=operation->getParentObject()->getName(true) + QString(".") + obj_name;
-	}
-	else
-	{
-		obj_type=ObjectType::BaseObject;
-		obj_name=tr("(invalid object)");
-	}
+	return operations[oper_idx];
 }
 
 unsigned OperationList::getChainSize()
@@ -575,10 +556,6 @@ void OperationList::undoOperation()
 		Operation *operation=nullptr;
 		bool chain_active=false;
 		Exception error;
-		unsigned chain_size=0, pos=0;
-
-		//if(!this->signalsBlocked())
-		chain_size=getChainSize();
 
 		do
 		{
@@ -603,8 +580,8 @@ void OperationList::undoOperation()
 
 			try
 			{
-				if(chain_size > 0 && operation->isOperationValid())
-					pos++;
+				//if(chain_size > 0 && operation->isOperationValid())
+				//	pos++;
 
 				//Executes the undo operation
 				executeOperation(operation, false);
@@ -634,9 +611,6 @@ void OperationList::redoOperation()
 		Operation *operation=nullptr;
 		bool chain_active=false;
 		Exception error;
-		unsigned chain_size=0, pos=0;
-
-		chain_size=getChainSize();
 
 		do
 		{
@@ -659,9 +633,6 @@ void OperationList::redoOperation()
 
 			try
 			{
-				if(chain_size > 0 && operation->isOperationValid())
-					pos++;
-
 				//Executes the redo operation (second argument as 'true')
 				executeOperation(operation, true);
 			}
@@ -768,20 +739,29 @@ void OperationList::executeOperation(Operation *oper, bool redo)
 			if(aux_obj)
 				oper->setXMLDefinition(orig_obj->getSourceCode(SchemaParser::XmlCode));
 
-
 			//For pk constraint, before restore the previous configuration, uncheck the not-null flag of the source columns
 			if(obj_type==ObjectType::Constraint)
 				dynamic_cast<Constraint *>(orig_obj)->setColumnsNotNull(false);
 
 			/* The original object (obtained from the table, relationship or model) will have its
-		previous values restored with the existing copy on the pool. After restoring the object
-		on the pool will have the same attributes as the object before being restored
-		to enable redo operations */
+			 * previous values restored with the existing copy on the pool. After restoring the object
+			 * on the pool will have the same attributes as the object before being restored
+			 * to enable redo operations */
+
+			/* When executing a ObjModified operation, we nee first to clear the original
+			 * object's dependencies to update them further */
+			if(op_type == Operation::ObjModified)
+				orig_obj->clearDependencies();
+
 			CoreUtilsNs::copyObject(reinterpret_cast<BaseObject **>(&bkp_obj), orig_obj, obj_type);
 			CoreUtilsNs::copyObject(reinterpret_cast<BaseObject **>(&orig_obj), object, obj_type);
 			CoreUtilsNs::copyObject(reinterpret_cast<BaseObject **>(&object), bkp_obj, obj_type);
 			object=orig_obj;
 
+			// Updating the original object dependencies after restoring the previous state
+			if(op_type != Operation::ObjMoved)
+				orig_obj->updateDependencies();
+
 			if(aux_obj)
 				CoreUtilsNs::copyObject(reinterpret_cast<BaseObject **>(&object), aux_obj, obj_type);
 
@@ -795,7 +775,7 @@ void OperationList::executeOperation(Operation *oper, bool redo)
 			the existing pool object will be inserted into table or in your relationship
 			on its original index */
 		else if((op_type==Operation::ObjRemoved && !redo) ||
-				(op_type==Operation::ObjCreated && redo))
+						(op_type==Operation::ObjCreated && redo))
 		{
 			if(aux_obj)
 				CoreUtilsNs::copyObject(reinterpret_cast<BaseObject **>(&object), aux_obj, obj_type);
@@ -929,14 +909,8 @@ void OperationList::executeOperation(Operation *oper, bool redo)
 			}
 			else if(obj_type==ObjectType::Tag)
 			{
-				std::vector<BaseObject *> refs;
-				model->getObjectReferences(object, refs);
-
-				while(!refs.empty())
-				{
-					dynamic_cast<BaseTable *>(refs.back())->setModified(true);
-					refs.pop_back();
-				}
+				for(auto &obj : object->getReferences())
+					dynamic_cast<BaseTable *>(obj)->setModified(true);
 			}
 		}
 
@@ -946,12 +920,9 @@ void OperationList::executeOperation(Operation *oper, bool redo)
 				 object->getObjectType()==ObjectType::Table || object->getObjectType()==ObjectType::ForeignTable ||
 				 object->getObjectType()==ObjectType::View || object->getObjectType()==ObjectType::Extension))
 		{
-			std::vector<BaseObject *> ref_objs;
-			model->getObjectReferences(object, ref_objs);
-
-			for(auto &obj : ref_objs)
+			for(auto &obj : object->getReferences())
 			{
-				if(obj->getObjectType()==ObjectType::Column)
+				if(obj->getObjectType() == ObjectType::Column)
 					dynamic_cast<Column *>(obj)->getParentTable()->setModified(true);
 			}
 		}
