@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -358,3 +358,16 @@ QString Role::getAlterCode(BaseObject *object)
 		throw Exception(e.getErrorMessage(),e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e);
 	}
 }
+
+void Role::updateDependencies()
+{
+	std::vector<BaseObject *> deps;
+
+	for(auto &rl : member_roles)
+		deps.push_back(rl);
+
+	for(auto &rl : admin_roles)
+		deps.push_back(rl);
+
+	BaseObject::updateDependencies(deps);
+}
