@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,31 +25,23 @@
 #ifndef VIEW_H
 #define VIEW_H
 
-#include "reference.h"
 #include "basetable.h"
+#include "trigger.h"
+#include "rule.h"
+#include "index.h"
+#include "genericsql.h"
 
 class __libcore View: public BaseTable {
 	private:
-		//! \brief Stores the references to expressions and objects
-		std::vector<Reference> references;
-
-		/*! \brief Vectors that stores indexes to the view references in each
-		 SQL part: SELECT-FROM, FROM-WHERE, after WHERE, expressions at the very end of definition (e.g. group by) */
-		std::vector<unsigned>	exp_select,
-		exp_from,
-		exp_where,
-		exp_end;
-
 		std::vector<TableObject *> triggers;
 		std::vector<TableObject *> rules;
 		std::vector<TableObject *> indexes;
 
-		std::vector<SimpleColumn> columns;
+		std::vector<SimpleColumn> gen_columns, custom_cols;
 
-		/*! \brief Commom table expression. This is prepend on the views definition.
-		CTE's are available since PostgreSQL 8.4:
-			> http://www.postgresql.org/docs/8.4/interactive/queries-with.html */
-		QString cte_expression;
+		std::vector<Reference> references;
+
+		QString sql_definition;
 
 		//! \brief Indicates that the view is a materialized one. This setting is auto exclusive with 'recursive'
 		bool materialized,
@@ -60,19 +52,6 @@ class __libcore View: public BaseTable {
 		//! \brief Indicates that the view is a a recursive one. This setting is auto exclusive with 'materialized'
 		recursive;
 
-		//! \brief Sets the definition attribute used by the SchemaParser
-		void setDefinitionAttribute();
-
-		//! \brief Sets the references attribute used by the SchemaParser
-		void setReferencesAttribute();
-
-		/*! \brief Returns the reference index on the view. When it doesn't exists,
-		 the method returns -1 */
-		int getReferenceIndex(Reference &refer);
-
-		//! \brief Returns the reference to internal expression list according to the SQL expression type
-		std::vector<unsigned> *getExpressionList(Reference::SqlType sql_type);
-
 		void setSQLObjectAttribute();
 
 		//! \brief Returns a unique name for a columns comparing it to the existent columns. In case of duplication the name receives a suffix
@@ -80,11 +59,12 @@ class __libcore View: public BaseTable {
 
 	public:
 		View();
+
 		virtual ~View();
 
-		void setName(const QString &name);
-		void setSchema(BaseObject *schema);
-		void setProtected(bool value);
+		virtual void setName(const QString &name) override;
+		virtual void setSchema(BaseObject *schema) override;
+		virtual void setProtected(bool value) override;
 
 		void setMaterialized(bool value);
 		void setRecursive(bool value);
@@ -94,20 +74,22 @@ class __libcore View: public BaseTable {
 		bool isRecursive();
 		bool isWithNoData();
 
+		void setReferences(const std::vector<Reference> &obj_refs);
+
+		void setCustomColumns(const std::vector<SimpleColumn> &cols);
+
+		void setSqlDefinition(const QString &sql_def);
+
+		QString getSqlDefinition();
+
 		/*! \brief Returns a simple column by searching by the name
 		 *  This method will return an invalid SimpleColumn instance if there's
 		 *  no matching column in the view */
 		SimpleColumn getColumn(const QString &name);
 
-		/*! \brief Adds a reference to the view specifying the SQL expression type for it
-		 (refer to class Reference::SQL_??? constants). The 'expr_id' parameter is the
-		 index where the reference must be inserted. By defaul the method always adds
-		 new references at the end of the list */
-		void addReference(Reference &refer, Reference::SqlType sql_type, int expr_id=-1);
-
 		/*! \brief Adds a trigger or rule into the view. If the index is specified ( obj_idx >= 0)
 		inserts the object at the position */
-		void addObject(BaseObject *obj, int obj_idx=-1);
+		virtual void addObject(BaseObject *obj, int obj_idx=-1) override;
 
 		//! \brief Adds a trigger into the view
 		void addTrigger(Trigger *trig, int obj_idx=-1);
@@ -119,13 +101,13 @@ class __libcore View: public BaseTable {
 		void addIndex(Index *index, int obj_idx=-1);
 
 		//! \brief Remove a object from view using its reference
-		void removeObject(BaseObject *obj);
+		virtual void removeObject(BaseObject *obj) override;
 
 		//! \brief Removes the object using the index and type
-		void removeObject(unsigned obj_idx, ObjectType obj_type);
+		virtual void removeObject(unsigned obj_idx, ObjectType obj_type) override;
 
 		//! \brief Removes the object using the name and type
-		void removeObject(const QString &name, ObjectType obj_type);
+		virtual void removeObject(const QString &name, ObjectType obj_type) override;
 
 		//! \brief Remove a trigger from view using its index
 		void removeTrigger(unsigned idx);
@@ -137,19 +119,19 @@ class __libcore View: public BaseTable {
 		void removeIndex(unsigned idx);
 
 		//! \brief Returns the object index searching by its reference
-		int getObjectIndex(BaseObject *obj);
+		virtual	int getObjectIndex(BaseObject *obj) override;
 
 		//! \brief Returns the object index searching by its index and type
-		int getObjectIndex(const QString &name, ObjectType obj_type);
+		virtual int getObjectIndex(const QString &name, ObjectType obj_type) override;
 
 		//! \brief Returns the children objects of the view excluding the provided children types (does not include references)
-		std::vector<BaseObject *> getObjects(const std::vector<ObjectType> &excl_types = {});
+		virtual std::vector<BaseObject *> getObjects(const std::vector<ObjectType> &excl_types = {}) override;
 
 		//! \brief Returns the view's child object using its index and type
-		TableObject *getObject(unsigned obj_idx, ObjectType obj_type);
+		virtual TableObject *getObject(unsigned obj_idx, ObjectType obj_type) override;
 
 		//! \brief Returns the view's child object using its name and type
-		TableObject *getObject(const QString &name, ObjectType obj_type);
+		virtual TableObject *getObject(const QString &name, ObjectType obj_type) override;
 
 		//! \brief Returns a trigger searching by its index
 		Trigger *getTrigger(unsigned obj_idx);
@@ -161,7 +143,7 @@ class __libcore View: public BaseTable {
 		Index *getIndex(unsigned obj_idx);
 
 		//! \brief Returns the view's child object count
-		unsigned getObjectCount(ObjectType obj_type, bool=false);
+		virtual unsigned getObjectCount(ObjectType obj_type, bool=false) override;
 
 		//! \brief Returns the view's trigger count
 		unsigned getTriggerCount();
@@ -178,40 +160,6 @@ class __libcore View: public BaseTable {
 		//! \brief Returns the object list according to specified type
 		std::vector<TableObject *> *getObjectList(ObjectType obj_type);
 
-		//! \brief Sets the commom table expression for the view
-		void setCommomTableExpression(const QString &expr);
-
-		/*! \brief Remove the reference from the view using its index, removing all the elements
-		 from the exp_??? vectors when they make use of the deleted reference. */
-		void removeReference(unsigned ref_id);
-
-		//! \brief Removes all the references from the view
-		void removeReferences();
-
-		//! \brief Removes an element from the expression list specified by the 'sql_type' parameter
-		void removeReference(unsigned expr_id, Reference::SqlType sql_type);
-
-		//! \brief Returns the commom table expression
-		QString getCommomTableExpression();
-
-		//! \brief Returns the reference count from view
-		unsigned getReferenceCount();
-
-		/*! \brief Returns the element count on the specified SQL expression type list (sql_type).
-		 It possible to filter the reference type via 'ref_type' which must be filled
-		 with the Reference::REFER_??? constants */
-		unsigned getReferenceCount(Reference::SqlType sql_type, int ref_type = -1);
-
-		//! \brief Returs one reference using its index
-		Reference getReference(unsigned ref_id);
-
-		/*! \brief Retuns one reference in the specified position (ref_id) on the
-		 specified expression list (sql_type) */
-		Reference getReference(unsigned ref_id, Reference::SqlType sql_type);
-
-		//! \brief Returns the specified reference index on the specified expression list
-		int getReferenceIndex(Reference &ref, Reference::SqlType sql_type);
-
 		//! \brief Returns the SQL / XML definition for the view
 		virtual QString getSourceCode(SchemaParser::CodeType def_type) final;
 
@@ -225,23 +173,17 @@ class __libcore View: public BaseTable {
 		bool isReferRelationshipAddedColumn();
 
 		/*! \brief Returns the list of all columns that is created by relationships.
-	This method is slower than isReferRelationshipAddedColumn() so it's not
-	recommended to use it only check if the object is referencing columns
-	added by relationship */
+		 * This method is slower than isReferRelationshipAddedColumn() so it's not
+		 * recommended to use it only check if the object is referencing columns
+		 * added by relationship */
 		std::vector<Column *> getRelationshipAddedColumns();
 
 		//! \brief Returns if the view is referencing the specified table
-		bool isReferencingTable(PhysicalTable *tab);
+		bool isReferencingTable(BaseTable *tab);
 
-		//! \brief Returns if the view is referencing the specified column
-		bool isReferencingColumn(Column *col);
+		virtual void setObjectListsCapacity(unsigned capacity) override;
 
-		//! \brief Returns if the view has an reference expression that is used as view definition
-		bool hasDefinitionExpression();
-
-		void setObjectListsCapacity(unsigned capacity);
-
-		unsigned getMaxObjectCount();
+		virtual unsigned getMaxObjectCount() override;
 
 		/*! \brief Returns a list of deduced names for view's colums (useful for recursive views).
 		 *	The names are retrieved, first, from columns aliases and lastly from table's columns
@@ -252,11 +194,21 @@ class __libcore View: public BaseTable {
 		//! \brief Returns the deduced columns of the view
 		std::vector<SimpleColumn> getColumns();
 
-		virtual QString getDataDictionary(bool split, const attribs_map &extra_attribs = {});
+		//! \brief Returns the user-defined columns of the view
+		std::vector<SimpleColumn> getCustomColumns();
+
+		std::vector<Reference> getObjectReferences();
+
+		std::vector<BaseTable *> getReferencedTables();
+
+		virtual QString getDataDictionary(bool split, const attribs_map &extra_attribs = {}) override;
+
 		virtual QString getAlterCode(BaseObject *object) final;
 
 		//! \brief Copy the attributes between two views
 		void operator = (View &visao);
+
+		virtual void updateDependencies() override;
 };
 
 #endif
