@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #define MODEL_FIX_FORM_H
 
 #include <QtWidgets>
-#include "globalattributes.h"
 #include "ui_modelfixform.h"
 #include "widgets/fileselectorwidget.h"
 
@@ -43,8 +42,9 @@ class __libgui ModelFixForm: public QDialog, public Ui::ModelFixForm {
 		*output_file_sel,
 		*pgmodeler_cli_sel;
 
-		void hideEvent(QHideEvent *);
+		void closeEvent(QCloseEvent *event);
 		void resetFixForm();
+		void enableFixOptions(bool enable);
 
 	public:
 		ModelFixForm(QWidget * parent = nullptr, Qt::WindowFlags f = Qt::Widget);
@@ -55,6 +55,7 @@ class __libgui ModelFixForm: public QDialog, public Ui::ModelFixForm {
 	private slots:
 		void enableFix();
 		void fixModel();
+		void cancelFix();
 		void updateOutput();
 		void handleProcessFinish(int res);
 
