@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,15 +23,24 @@ FileSelectorWidget::FileSelectorWidget(QWidget *parent) : QWidget(parent)
 {
 	setupUi(this);
 	allow_filename_input = read_only = false;
-
-	file_dlg.setWindowIcon(QPixmap(GuiUtilsNs::getIconPath("pgmodeler_logo")));
+	file_is_mandatory = check_exec_flag = false;
+	file_must_exist = false;
+	file_mode = QFileDialog::AnyFile;
 
 	filename_edt->setReadOnly(true);
 	filename_edt->installEventFilter(this);
 
 	warn_ico_lbl = new QLabel(this);
 	warn_ico_lbl->setVisible(false);
-	warn_ico_lbl->setMinimumSize(filename_edt->height() * 0.75, filename_edt->height() * 0.75);
+
+	#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+		int sz = filename_edt->style()->pixelMetric(QStyle::PM_LineEditIconSize) +
+				(filename_edt->style()->pixelMetric(QStyle::PM_LineEditIconMargin)/2);
+	#else
+		int sz = filename_edt->height() * 0.75;
+	#endif
+
+	warn_ico_lbl->setMinimumSize(sz, sz);
 	warn_ico_lbl->setMaximumSize(warn_ico_lbl->minimumSize());
 	warn_ico_lbl->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 	warn_ico_lbl->setScaledContents(true);
@@ -42,7 +51,6 @@ FileSelectorWidget::FileSelectorWidget(QWidget *parent) : QWidget(parent)
 	connect(rem_file_tb, &QToolButton::clicked, this, &FileSelectorWidget::clearSelector);
 
 	connect(filename_edt, &QLineEdit::textChanged, this, [this](const QString &text){
-		rem_file_tb->setEnabled(!text.isEmpty());
 		validateSelectedFile();
 		emit s_selectorChanged(!text.isEmpty());
 	});
@@ -66,7 +74,14 @@ bool FileSelectorWidget::eventFilter(QObject *obj, QEvent *evnt)
 void FileSelectorWidget::resizeEvent(QResizeEvent *)
 {
 	warn_ico_lbl->move(filename_edt->width() - warn_ico_lbl->width() - 5,
-										 (filename_edt->height() - warn_ico_lbl->height()) / 2);
+
+	#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+		filename_edt->style()->pixelMetric(QStyle::PM_LineEditIconMargin) + 1
+	#else
+		(filename_edt->height() - warn_ico_lbl->height()) / 2
+	#endif
+
+	);
 }
 
 void FileSelectorWidget::setAllowFilenameInput(bool allow_fl_input)
@@ -75,29 +90,60 @@ void FileSelectorWidget::setAllowFilenameInput(bool allow_fl_input)
 	filename_edt->setReadOnly(!allow_filename_input);
 }
 
-void FileSelectorWidget::setFileMode(QFileDialog::FileMode file_mode)
+void FileSelectorWidget::setDirectoryMode(bool dir_mode)
 {
-	// Forcing the ExistingFile (single file selection) if multiple file selection is provided
-	if(file_mode == QFileDialog::ExistingFiles)
-		file_mode = QFileDialog::ExistingFile;
+	if(dir_mode)
+		file_mode = QFileDialog::Directory;
+	else
+	{
+		if(file_must_exist)
+			file_mode = QFileDialog::ExistingFile;
+		else
+			file_mode = QFileDialog::AnyFile;
+	}
 
-	file_dlg.setFileMode(file_mode);
 	validateSelectedFile();
 }
 
 void FileSelectorWidget::setAcceptMode(QFileDialog::AcceptMode accept_mode)
 {
-	file_dlg.setAcceptMode(accept_mode);
+	this->accept_mode = accept_mode;
 }
 
 void FileSelectorWidget::setNameFilters(const QStringList &filters)
 {
-	file_dlg.setNameFilters(filters);
+	name_filters = filters;
+}
+
+void FileSelectorWidget::setNamePattern(const QString &pattern)
+{
+	name_regexp.setPattern(pattern);
+}
+
+void FileSelectorWidget::setCheckExecutionFlag(bool value)
+{
+	check_exec_flag = value;
+}
+
+void FileSelectorWidget::setFileIsMandatory(bool value)
+{
+	file_is_mandatory = value;
+	validateSelectedFile();
+}
+
+void FileSelectorWidget::setFileMustExist(bool value)
+{
+	file_must_exist = value;
+
+	if(file_must_exist && file_mode == QFileDialog::AnyFile)
+		file_mode = QFileDialog::ExistingFile;
+
+	validateSelectedFile();
 }
 
 void FileSelectorWidget::setFileDialogTitle(const QString &title)
 {
-	file_dlg.setWindowTitle(title);
+	file_dlg_title = title;
 }
 
 void FileSelectorWidget::setSelectedFile(const QString &file)
@@ -107,22 +153,38 @@ void FileSelectorWidget::setSelectedFile(const QString &file)
 
 void FileSelectorWidget::setMimeTypeFilters(const QStringList &filters)
 {
-	file_dlg.setMimeTypeFilters(filters);
+	mime_filters = filters;
 }
 
 void FileSelectorWidget::setDefaultSuffix(const QString &suffix)
 {
-	file_dlg.setDefaultSuffix(suffix);
+	def_suffix = suffix;
+}
+
+void FileSelectorWidget::setAppendSuffix(bool append)
+{
+	append_suffix = append;
 }
 
 bool FileSelectorWidget::hasWarning()
 {
-	QString str = warn_ico_lbl->toolTip();
 	return !warn_ico_lbl->toolTip().isEmpty();
 }
 
 QString FileSelectorWidget::getSelectedFile()
 {
+	if(append_suffix && allow_filename_input &&
+		 file_mode != QFileDialog::Directory &&
+		 !def_suffix.isEmpty())
+	{
+		QString filename = filename_edt->text();
+
+		if(QFileInfo(filename).completeSuffix().isEmpty())
+			filename.append("." + def_suffix);
+
+		return filename;
+	}
+
 	return filename_edt->text();
 }
 
@@ -166,8 +228,17 @@ void FileSelectorWidget::setCustomWarning(const QString &warn_msg)
 
 void FileSelectorWidget::openFileDialog()
 {
+	QFileDialog file_dlg;
+
 	filename_edt->clearFocus();
+	file_dlg.setWindowIcon(QIcon(GuiUtilsNs::getIconPath("pgmodeler_logo")));
 	file_dlg.selectFile(filename_edt->text());
+	file_dlg.setFileMode(file_mode);
+	file_dlg.setAcceptMode(accept_mode);
+	file_dlg.setNameFilters(name_filters);
+	file_dlg.setWindowTitle(file_dlg_title);
+	file_dlg.setMimeTypeFilters(mime_filters);
+	file_dlg.setDefaultSuffix(def_suffix);
 
 	GuiUtilsNs::restoreFileDialogState(&file_dlg);
 	file_dlg.exec();
@@ -187,7 +258,7 @@ void FileSelectorWidget::openFileExternally()
 
 void FileSelectorWidget::showWarning()
 {
-	QPalette pal;
+	QColor color = qApp->palette().color(QPalette::Text);
 	int padding = 0;
 	bool has_warn = !warn_ico_lbl->toolTip().isEmpty();
 
@@ -195,34 +266,55 @@ void FileSelectorWidget::showWarning()
 
 	if(has_warn)
 	{
-		pal.setColor(QPalette::Text, QColor(255, 0, 0));
+		color.setRgb(255, 0, 0);
 		padding = warn_ico_lbl->width();
 	}
-	else
-		pal = qApp->palette();
 
-	filename_edt->setStyleSheet(QString("padding: 2px %1px 2px 1px").arg(padding));
-	filename_edt->setPalette(pal);
+	filename_edt->setStyleSheet(QString("padding: 2px %1px 2px 1px; color: %2")
+															.arg(QString::number(padding), color.name()));
 }
 
 void FileSelectorWidget::validateSelectedFile()
 {
 	QFileInfo fi(filename_edt->text());
-	warn_ico_lbl->setToolTip("");
 
-	if(!filename_edt->text().isEmpty())
+	warn_ico_lbl->setToolTip("");
+	rem_file_tb->setEnabled(!filename_edt->text().isEmpty());
+
+	if((file_is_mandatory && fi.absoluteFilePath().isEmpty()) ||
+		 (!fi.absoluteFilePath().isEmpty() && !fi.isAbsolute()))
 	{
-		if(fi.exists() && fi.isDir() && file_dlg.fileMode() != QFileDialog::Directory)
+		if(file_mode == QFileDialog::Directory)
+			warn_ico_lbl->setToolTip(tr("An absolute path to a directory must be provided!"));
+		else
+			warn_ico_lbl->setToolTip(tr("An absolute path to a file must be provided!"));
+	}
+	else if(!fi.absoluteFilePath().isEmpty())
+	{
+		if(fi.exists() && fi.isDir() && file_mode != QFileDialog::Directory)
 			warn_ico_lbl->setToolTip(tr("The provided path is not a file!"));
-		else if(fi.exists() && fi.isFile() && file_dlg.fileMode() == QFileDialog::Directory)
+		else if(fi.exists() && fi.isFile() && file_mode == QFileDialog::Directory)
 			warn_ico_lbl->setToolTip(tr("The provided path is not a directory!"));
-		else if(!fi.exists() && file_dlg.fileMode() != QFileDialog::AnyFile)
+		else if(file_must_exist && !fi.exists())
 		{
-			if(file_dlg.fileMode() == QFileDialog::Directory)
+			if(file_mode == QFileDialog::Directory)
 				warn_ico_lbl->setToolTip(tr("No such directory!"));
 			else
 				warn_ico_lbl->setToolTip(tr("No such file!"));
 		}
+		else if(fi.exists())
+		{
+			// Validating the name pattern against the selected filename
+			if(name_regexp.isValid() && !fi.absoluteFilePath().contains(name_regexp))
+			{
+				if(file_mode == QFileDialog::Directory)
+					warn_ico_lbl->setToolTip(tr("The selected directory is not valid!"));
+				else
+					warn_ico_lbl->setToolTip(tr("The selected file is not valid!"));
+			}
+			else if(check_exec_flag && !fi.isDir() && !fi.isExecutable())
+				warn_ico_lbl->setToolTip(tr("The selected file is not executable!"));
+		}
 	}
 
 	showWarning();
