@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2024 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,18 +24,22 @@ namespace Attributes {
 	ActiveLayers("active-layers"),
 	AdminOption("admin-option"),
 	AdminRoles("admin-roles"),
+	AlertUnsavedModels("alert-unsaved-models"),
+	AlertOpenSqlTabs("alert-open-sqltabs"),
 	Alias("alias"),
 	Alignment("alignment"),
 	AlignObjsToGrid("align-objs-to-grid"),
+	AllowCompletion("allow-completion"),
 	AllowConns("allow-conns"),
 	AllVersions("all"),
 	AlterCmds("alter-cmds"),
 	AnalyzeFunc("analyze"),
 	AncestorTable("ancestor-table"),
+	Any("any"),
 	AppendAtEod("append-at-eod"),
 	AppendedSql("appended-sql"),
 	Application("application"),
-	ApplyOnServer="apply-on-server",
+	ApplyOnServer("apply-on-server"),
 	ArgCount("arg-count"),
 	ArgDefaults("arg-defaults"),
 	ArgDefCount("arg-def-count"),
@@ -59,6 +63,7 @@ namespace Attributes {
 	Bold("bold"),
 	BorderColor("border-color"),
 	Bottom("bottom"),
+	BlogPost("blogpost"),
 	Buffering("buffering"),
 	BypassRls("bypassrls"),
 	ByValue("by-value"),
@@ -78,8 +83,9 @@ namespace Attributes {
 	CheckVersions("check-versions"),
 	CkConstr("ck-constr"),
 	ClientEncoding("client-encoding"),
+	CloseChar("close-char"),
 	Code("code"),
-	CodeCompletion("code-completion"),	
+	CodeCompletion("code-completion"),
 	ColIndexes("col-indexes"),
 	ColIsIdentity("col-is-identity"),
 	CollapseMode("collapse-mode"),
@@ -91,13 +97,14 @@ namespace Attributes {
 	ColsComment("cols-comment"),
 	Column("column"),
 	ColumnAlias("column-alias"),
+	ColumnTruncThreshold("column-trunc-threshold"),
 	Columns("columns"),
 	Command("command"),
 	Commands("commands"),
 	Comment("comment"),
 	CommutatorOp("commutator-op"),
 	CompactView("compact-view"),
-	CompareToDatabase="compare-to-db",
+	CompareToDatabase("compare-to-db"),
 	ComparisonType("comparison-type"),
 	CompletionTrigger("completion-trigger"),
 	CompositeType("composite"),
@@ -134,7 +141,7 @@ namespace Attributes {
 	CrowsFoot("crows-foot"),
 	CteExpression("cte-exp"),
 	CurIdentityType("cur-identity-type"),
-	CurrentModel="current-model",
+	CurrentModel("current-model"),
 	CurVersion("cur-version"),
 	CustomColor("custom-color"),
 	CustomFilter("custom-filter"),
@@ -174,15 +181,16 @@ namespace Attributes {
 	Dimension("dimension"),
 	Directory("directory"),
 	DisableSmoothness("disable-smoothness"),
+	DisableInlineEditor("disable-inline-editor"),
 	DisplayLineNumbers("display-line-numbers"),
 	DockWidgets("dock-widgets"),
 	DomConstraint("domconstraint"),
-	DontDropMissingObjs="dont-drop-missing-objs",
+	DontDropMissingObjs("dont-drop-missing-objs"),
 	Dot("dot"),
 	Drop("drop"),
 	DropCmds("drop-cmds"),
-	DropMissingColsConstrs="drop-missing-cols-constrs",
-	DropTruncCascade="drop-trunc-cascade",
+	DropMissingColsConstrs("drop-missing-cols-constrs"),
+	DropTruncCascade("drop-trunc-cascade"),
 	DstColPattern("dst-col-pattern"),
 	DstColumns("dst-columns"),
 	DstEncoding("dst-encoding"),
@@ -198,6 +206,7 @@ namespace Attributes {
 	ElementsCount("elements-count"),
 	EmptyPassword("empty-password"),
 	Encoding("encoding"),
+	EnclosingChars("enclosing-chars"),
 	Encrypted("encrypted"),
 	EndExp("end-exp"),
 	EnumType("enumeration"),
@@ -214,6 +223,7 @@ namespace Attributes {
 	ExecutionCost("execution-cost"),
 	ExecutPriv("execute"),
 	ExistingValue("existing-value"),
+	ExpansionFactor("expansion-factor"),
 	Explicit("explicit"),
 	Export("export"),
 	ExportToFile("export-to-file"),
@@ -223,6 +233,7 @@ namespace Attributes {
 	ExtAttribsPerPage("ext-attribs-per-page"),
 	ExtObjOids("ext-obj-oids"),
 	ExtraCondition("extra-condition"),
+	ExtraInfo("extra-info"),
 	Factor("factor"),
 	FadedOut("faded-out"),
 	FadeInObjects("fadein-objects"),
@@ -244,7 +255,7 @@ namespace Attributes {
 	FkDefs("fk-defs"),
 	Font("font"),
 	FontSize("font-size"),
-	ForceObjsRecreation="force-objs-recreation",
+	ForceObjsRecreation("force-objs-recreation"),
 	ForegroundColor("foreground-color"),
 	ForeignKeys("foreign-keys"),
 	ForeignTableTogglerButtons("foreigntable-toggler-btns"),
@@ -269,12 +280,13 @@ namespace Attributes {
 	GridPattern("grid-pattern"),
 	Group("group"),
 	HandlerFunc("handler"),
-	HandlesType("handles-type"),
 	HasChanges("has-changes"),
 	Hashes("hashes"),
 	HbaFile("hba-file"),
 	Height("height"),
+	HideDescription("hide-description"),
 	HideExtAttribs("hide-ext-attribs"),
+	HideObjShadows("hide-obj-shadows"),
 	HideRelName("hide-rel-name"),
 	HideSchNameUserTypes("hide-schema-names-of-types"),
 	HideTableTags("hide-table-tags"),
@@ -288,13 +300,13 @@ namespace Attributes {
 	Identifier("identifier"),
 	IdentityType("identity-type"),
 	IgnoredChars("ignored-chars"),
-	IgnoreDuplicErrors="ignore-duplic-errors",
-	IgnoreErrorCodes="ignore-error-codes",
-	IgnoreImportErrors="ignore-import-errors",
+	IgnoreDuplicErrors("ignore-duplic-errors"),
+	IgnoreErrorCodes("ignore-error-codes"),
+	IgnoreImportErrors("ignore-import-errors"),
 	Implicit("implicit"),
 	Import("import"),
-	ImportExtObjs="import-ext-objs",
-	ImportSysObjs="import-sys-objs",
+	ImportExtObjs("import-ext-objs"),
+	ImportSysObjs("import-sys-objs"),
 	Increment("increment"),
 	Index("index"),	
 	IndexElement("idxelement"),
@@ -309,8 +321,9 @@ namespace Attributes {
 	InitialCond("initial-cond"),
 	InitialData("initial-data"),
 	InitialExp("initial-exp"),
+	InkSaver("inksaver"),
 	InlineFunc("inline"),
-	InputDatabase="input-db",
+	InputDatabase("input-db"),
 	InputFunc("input"),
 	InsertPriv("insert"),
 	InsEvent("ins-event"),
@@ -318,14 +331,15 @@ namespace Attributes {
 	IntervalType("interval-type"),
 	InvertRangeSelTrigger("invert-rangesel-trigger"),
 	IoCast("io-cast"),
+	IsExtType("is-ext-type"),
 	IsPartitioned("is-partitioned"),
 	IsTemplate("is-template"),
 	Italic("italic"),
 	Item("item"),
 	Items("items"),
 	JoinFunc("join"),
-	KeepClusterObjs="keep-cluster-objs",
-	KeepObjsPerms="keep-objs-perms",
+	KeepClusterObjs("keep-cluster-objs"),
+	KeepObjsPerms("keep-objs-perms"),
 	Label("label"),
 	Labels("labels"),
 	LabelsPos("labels-pos"),
@@ -383,6 +397,7 @@ namespace Attributes {
 	Mode("mode"),
 	ModelAuthor("author"),
 	Name("name"),
+	NameColor("name-color"),
 	NameFilter("name-filter"),
 	NameLabel("name-label"),
 	NamePatterns("name-patterns"),
@@ -399,12 +414,14 @@ namespace Attributes {
 	NotExtObject("not-ext-object"),
 	NotNull("not-null"),
 	NullsFirst("nulls-first"),
+	NullsNotDistinct("nulls-not-distinct"),
 	Object("object"),
 	ObjectFinder("objectfinder"),
 	ObjectId("object-id"),
 	Objects("objects"),
 	ObjectType("object-type"),
 	ObjCount("objcount"),
+	ObjRelationName("obj-rel-name"),
 	ObjSelection("obj-selection"),
 	ObjShadow("obj-shadow"),
 	Oid("oid"),
@@ -416,6 +433,7 @@ namespace Attributes {
 	OldVersion("old-version"),
 	OpClass("opclass"),
 	OpClasses("opclasses"),
+	OpenChar("open-char"),
 	Operator("operator"),
 	OperatorFunc("operfunc"),
 	Operators("operators"),
@@ -493,7 +511,7 @@ namespace Attributes {
 	RangeType("range"),
 	Recent("recent"),
 	RecentModels("recent-models"),
-	RecreateUnmodObjs="recreate-unmod-objs",
+	RecreateUnmodObjs("recreate-unmod-objs"),
 	RectVisible("rect-visible"),
 	Recursive("recursive"),
 	RecvFunc("receive"),
@@ -544,6 +562,7 @@ namespace Attributes {
 	Rules("rules"),
 	SaveLastPosition("save-last-position"),
 	SaveRestoreGeometry("save-restore-geometry"),
+	SceneRect("scene-rect"),
 	Schema("schema"),
 	SchemaOid("schema-oid"),
 	SearchPath("search-path"),
@@ -571,6 +590,7 @@ namespace Attributes {
 	ShowPageDelimiters("show-page-delimiters"),
 	ShowSourcePane("show-source-pane"),
 	Signature("signature"),
+	SimpleCol("simplecol"),
 	SimpleExp("simple-exp"),
 	SimplifiedObjCreation("simplified-obj-creation"),
 	SinglePkColumn("single-pk-col"),
@@ -609,7 +629,7 @@ namespace Attributes {
 	StateType("state-type"),
 	Storage("storage"),
 	StorageParams("stg-params"),
-	StoreInFile="store-in-file",
+	StoreInFile("store-in-file"),
 	StrategyNum("stg-number"),
 	Stikeout("strikeout"),
 	Style("style"),
@@ -619,7 +639,6 @@ namespace Attributes {
 	Superuser("superuser"),
 	Symbol("symbol"),
 	System("system"),
-	SyntaxHlTheme("syntax-hl-theme"),
 	Table("table"),
 	TableTogglerButtons("table-toggler-btns"),
 	TableTogglerBody("table-toggler-body"),
@@ -646,6 +665,7 @@ namespace Attributes {
 	Triggers("triggers"),
 	True("true"),
 	Truncate("truncate"),
+	TruncateColumnData("truncate-column-data"),
 	TruncatePriv("truncate"),
 	TruncEvent("trunc-event"),
 	Trusted("trusted"),
@@ -674,7 +694,9 @@ namespace Attributes {
 	UqPattern("uq-pattern"),
 	UsagePriv("usage"),
 	UseChangelog("use-changelog"),
+	UseColumns("use-columns"),
 	UseCurvedLines("use-curved-lines"),
+	UseDefDisambiguation("use-def-disambiguation"),
 	UsePlaceholders("use-placeholders"),
 	UseSignature("use-signature"),
 	UseSorting("use-sorting"),
